from mpf.tests.MpfTestCase import MpfTestCase


class TestSwitchController(MpfTestCase):

    def getConfigFile(self):
        return 'config.yaml'

    def getMachinePath(self):
        return 'tests/machine_files/switch_controller/'

    def _callback(self):
         self.isActive = self.machine.switch_controller.is_active("s_test", ms=300)

    def test_is_active_timing(self):
        self.isActive = None

        self.machine.switch_controller.add_switch_handler(
                switch_name="s_test",
                callback=self._callback,
                state=1, ms=300)
        self.machine.switch_controller.process_switch("s_test", 1, True)

        self.advance_time_and_run(3)

        self.assertEqual(True, self.isActive)

    def test_initial_state(self):
        # tests that when MPF starts, the initial states of switches that
        # started in that state are read correctly.
        self.assertFalse(self.machine.switch_controller.is_active('s_test',
                                                                  1000))

    def _callback_invalid(self):
         raise AssertionError("Should not be called")

    def test_timed_switch_handler(self):
        self.machine.switch_controller.process_switch("s_test", 1)
        self.advance_time_and_run(3)

        self.machine.switch_controller.add_switch_handler(
                switch_name="s_test",
                callback=self._callback_invalid,
                state=0, ms=250)

        self.machine.switch_controller.process_switch("s_test", 0)
        self.advance_time_and_run(.1)

        self.machine.switch_controller.remove_switch_handler(
                switch_name="s_test",
                callback=self._callback_invalid,
                state=0, ms=250)

        self.advance_time_and_run(5)

    def test_activation_and_deactivation_events(self):
        self.mock_event("test_active")
        self.mock_event("test_active2")
        self.mock_event("test_inactive")
        self.mock_event("test_inactive2")

        self.machine.switch_controller.process_switch("s_test_events", 1)
        self.machine_run()

        self.assertEqual(0, self._events['test_active'])
        self.assertEqual(1, self._events['test_active2'])
        self.assertEqual(0, self._events['test_inactive'])
        self.assertEqual(0, self._events['test_inactive2'])

        self.advance_time_and_run(1)

        self.assertEqual(1, self._events['test_active'])
        self.assertEqual(1, self._events['test_active2'])
        self.assertEqual(0, self._events['test_inactive'])
        self.assertEqual(0, self._events['test_inactive2'])

        self.machine.switch_controller.process_switch("s_test_events", 1)
        self.advance_time_and_run(1)

        self.assertEqual(1, self._events['test_active'])
        self.assertEqual(1, self._events['test_active2'])
        self.assertEqual(0, self._events['test_inactive'])
        self.assertEqual(0, self._events['test_inactive2'])

        self.machine.switch_controller.process_switch("s_test_events", 0)
        self.advance_time_and_run(1)

        self.assertEqual(1, self._events['test_active'])
        self.assertEqual(1, self._events['test_active2'])
        self.assertEqual(1, self._events['test_inactive'])
        self.assertEqual(0, self._events['test_inactive2'])

        self.advance_time_and_run(1)
        self.assertEqual(1, self._events['test_active'])
        self.assertEqual(1, self._events['test_active2'])
        self.assertEqual(1, self._events['test_inactive'])
        self.assertEqual(1, self._events['test_inactive2'])

    def test_ignore_window_ms(self):
        # first hit. switch gets active
        self.machine.switch_controller.process_switch("s_test_window_ms", 1)
        self.advance_time_and_run(.001)
        self.assertTrue(self.machine.switch_controller.is_active("s_test_window_ms"))

        # an disables instantly
        self.machine.switch_controller.process_switch("s_test_window_ms", 0)
        self.advance_time_and_run(.001)
        self.assertFalse(self.machine.switch_controller.is_active("s_test_window_ms"))

        # and enables again
        self.machine.switch_controller.process_switch("s_test_window_ms", 1)
        self.advance_time_and_run(.01)
        self.assertFalse(self.machine.switch_controller.is_active("s_test_window_ms"))
        self.advance_time_and_run(.05)
        self.assertFalse(self.machine.switch_controller.is_active("s_test_window_ms"))
        self.advance_time_and_run(.05)
        self.assertTrue(self.machine.switch_controller.is_active("s_test_window_ms"))
